#!/bin/bash
# check_tcp_connections - nagios plugin to check tcp connections
# status via ss utility
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

SS=/bin/ss
AWK=/usr/bin/awk

# No need to change anything below this line
#############################################
established=0
listen=0
synsent=0
synrecv=0
finw1=0
finw2=0
closew=0
timew=0
lastack=0
closing=0

ST_OK=0
ST_WR=1
ST_CR=2
ST_UK=3

PROGNAME=`basename $0`
VERSION="Version 1.0,"
AUTHOR="2013, Pavel aka Rascal (http://rascal.su/)"

wcdiff_req=0
wclvls_req=0

print_version() {
    echo "$VERSION $AUTHOR"
}

print_help() {
    print_version $PROGNAME $VERSION
    echo ""
    echo "Description:"
    echo "$PROGNAME is a Nagios plugin to check tcp connections status."
    echo ""
    echo "Example call:"
    echo "./$PROGNAME [-we] 4096 [-ce] 8192"
    echo ""
    echo "Options:"
    echo "  --warning-estab|-we)"
    echo "    Sets a warning level for established connections. Default is: off"
    echo "  --critical-estab|-ce)"
    echo "    Sets a critical level for established connections. Default is: off"
    exit $ST_UK
}

val_wcdiff_req() {
    if [ ! -z "$warn_estab" -a ! -z "$crit_estab" ]
    then
        wclvls_req=1

        if [ ${warn_estab} -gt ${crit_estab} ]
        then
            wcdiff_req=1
        fi
    elif [ ! -z "$warn_estab" -a -z "$crit_estab" ]
    then
        wcdiff_req=2
    elif [ -z "$warn_estab" -a ! -z "$crit_estab" ]
    then
        wcdiff_req=3
    fi
}

get_vals() {
    eval $($SS -ant | $AWK "{if (NR>1) {state[\$1]++}} END { \
	for (i in state) {s=i; \
	sub (/ESTAB/, \"established\", s); sub (/LISTEN/, \"listen\", s); \
	sub (/SYN-SENT/, \"synsent\", s);  sub (/SYN-RECV/, \"synrecv\", s); \
	sub (/FIN-WAIT-1/, \"finw1\", s); sub (/FIN-WAIT-2/, \"finw2\", s); \
	sub (/CLOSE-WAIT/, \"closew\", s); sub (/TIME-WAIT/, \"timew\", s); \
	sub (/LAST-ACK/, \"lastack\", s); sub (/CLOSING/, \"closing\", s); \
	print s\"=\"state[i]}}")
}

do_perfdata() {
    perfdata="'established'=$established 'listen'=$listen 'synsent'=$synsent \
'synrecv'=$synrecv 'finw1'=$finw1 'finw2'=$finw2 'closew'=$closew 'timew'=$timew \
'lastack'=$lastack 'closing'=$closing"
}

do_output() {
    output="Now have $listen listening ports and $established established connections."
}

while test -n "$1"; do
    case "$1" in
        --help|-h)
            print_help
            exit $ST_UK
            ;;
        --version|-v)
            print_version $PROGNAME $VERSION
            exit $ST_UK
            ;;
        --warning-estab|-we)
            warn_estab=$2
            shift
            ;;
        --critical-estab|-ce)
            crit_estab=$2
            shift
            ;;
        *)
            echo "Unknown argument: $1"
            print_help
            exit $ST_UK
            ;;
    esac
    shift
done

# Start
val_wcdiff_req

if [ "$wcdiff_req" = 1 ]
then
    echo "Please adjust your warning/critical thresholds. The warning must \
be lower than the critical level!"
    exit $ST_UK
elif [ "$wcdiff_req" = 2 ]
then
    echo "Please also set a critical value when you want to use \
warning/critical thresholds!"
    exit $ST_UK
elif [ "$wcdiff_req" = 3 ]
then
    echo "Please also set a warning value when you want to use \
warning/critical thresholds!"
    exit $ST_UK
else
    get_vals
    do_perfdata
    do_output

    if [ ${wclvls_req} = 1 ]
    then
        if [ ${established} -ge ${warn_estab} -a ${established} -lt ${crit_estab} ]
        then
            echo "WARNING - ${output} | ${perfdata}"
            exit $ST_WR
        elif [ ${established} -ge ${crit_estab} ]
        then
            echo "CRITICAL - ${output} | ${perfdata}"
        exit $ST_CR
        else
            echo "OK - ${output} | ${perfdata}"
            exit $ST_OK
        fi
    else
        echo "OK - ${output} | ${perfdata}"
        exit $ST_OK
    fi
fi
